from ._client import Client, Connection
from ._types import (
    Message,
    MessageId,
    MessageInfo,
    NewsItem,
    NewsItemId,
    NewsItemInfo,
    Person,
    Pupil,
    PupilId,
    ReplyMessage,
)

__all__ = [
    'Client',
    'Connection',
    'Message',
    'MessageId',
    'MessageInfo',
    'NewsItem',
    'NewsItemId',
    'NewsItemInfo',
    'Person',
    'Pupil',
    'PupilId',
    'ReplyMessage',
]
